if not BGControlPublic then BGControlPublic = {} end
if not BG_Control_SV then BG_Control_SV = {} end
if not BG_Control_SV.Options then BG_Control_SV.Options = {} end

if not BG_Control_SV.TalentCache then BG_Control_SV.TalentCache = {} end
if not BG_Control_SV.MMRCache then BG_Control_SV.MMRCache = {} end
if not BG_Control_SV.NodeCache then BG_Control_SV.NodeCache = {} end


-- Local hidden frames, functions, and everything else
local BGC = {
	frame = CreateFrame("Frame","BGC_Frame"),
	};

-- The "live" Player Database for where people are located and who did what
local PD;

function BGC.Player_Entering_Battleground(...)
	local _, type, _, _, _, _, _, mapID = GetInstanceInfo()
	if "pvp" == type then
		BGC.frame:RegisterEvent("UPDATE_BATTLEFIELD_SCORE");
		BGC.frame:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
		BGC.frame:RegisterEvent("INSTANCE_GROUP_SIZE_CHANGED");	
		BGC.InitNodeCache(mapID)
	end
end

function BGC.BGMapID()
	local _, type, _, _, _, _, _, mapID = GetInstanceInfo()
	return mapID
end

function BGC.InitNodeCache(mapID)
	local map = BG_Control_SV.NodeCache[mapID];
	if not map then
		map = {}
		map[
	end
end

function BGC.InitPD()
	PD={}
	PD.raidID = {}
	PD.damageMap = {}
	PD.healingMap = {}
end

function BGControlPublic.LeaveBattlefield(...)
	BGC.frame:UnregisterEvent("UPDATE_BATTLEFIELD_SCORE");
	BGC.frame:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
	BGC.frame:UnregisterEvent("INSTANCE_GROUP_SIZE_CHANGED");
	
	-- TODO: Save off talent data to the saved variables
	BGC.InitPD()
end

function BGC.Update_Battlefield_Score(...)
	--TODO: Use this to try to "prioritize" healers
end

function BGC.Combat_Log_Event_Unfiltered(...)
	local timeStamp, event, _, sourceGUID, sourceName, sourceFlags, _, destGUID, destName, destFlags, _ = ...
	
	if "SWING_DAMAGE" = event then
		BGC.SwingDamageDone(timeStamp,sourceGUID,sourceFlags,destGUID,destFlags)
	else if "RANGE_DAMAGE" = event then
		local spellID= select(12,...)
		BGC.SpellDamageDone(timeStamp,sourceGUID,sourceFlags,destGUID,destFlags,spellID)
	else if "SPELL_DAMAGE" = event then
		local spellID= select(12,...)
		BGC.SpellDamageDone(timeStamp,sourceGUID,sourceFlags,destGUID,destFlags,spellID)
	else if "SPELL_HEAL" = event then
		local spellID= select(12,...)
		BGC.HealCast(timeStamp,sourceGUID,sourceFlags,destGUID,destFlags,spellID)
	end

end

function BGC.Instance_Group_Size_Changed(...)
	for i=1,40 do
		local raidID = "raid"..i;
		local name, realm = UnitName(raidID)
		PD.raidID[BGC.PlayerName(name,realm)] = raidID
	end
end

function BGC.SwingDamageDone(time,from,fromFlags,to,toFlags)
	local fromName, toName

	if BGC.IsHostile(from) then
		fromName = BGC.PlayerNameFromGUID(from)
		toName = BGC.PlayerNameFromGUID(to)
	else if BGC.IsHostile(to) then
		fromName = BGC.PlayerNameFromGUID(to)
		toName = BGC.PlayerNameFromGUID(from)
	end

	local map = PD.damageMap[fromName]
	local x, y = UnitPosition(BGC.RaidID(toName))
	map.time = time
	map.range = 0
	map.target = toName
	map.lastSeen = { x , y }
	BGC.UpdateDamage(map)
end

function BGC.SpellDamageDone(time,from,fromFlags,to,toFlags,spellID)
	local fromName, toName

	if BGC.IsHostile(from) then
		fromName = BGC.PlayerNameFromGUID(from)
		toName = BGC.PlayerNameFromGUID(to)
	else if BGC.IsHostile(to) then
		fromName = BGC.PlayerNameFromGUID(to)
		toName = BGC.PlayerNameFromGUID(from)
	end

	local map = PD.damageMap[fromName]
	map.time = time
	map.range = BGC.SpellRange(spellID);
	map.target = toName
	local x, y = UnitPosition(BGC.RaidID(toName))
	map.lastSeen = { x , y }
	BGC.UpdateDamage(map)
end

function BGC.HealCast(time,from,fromFlags,to,toFlags,spellID)
	if BGC.IsHostile(fromFlags) then
		local map = PD.healingMap[fromName]
		local fromName = BGC.PlayerNameFromGUID(from)
		local toName = BGC.PlayerNameFromGUID(to)

		map.time = time
		map.range = BGC.SpellRange(spellID);
		map.target=toName;
		BGC.UpdateHealer(map)
	end
end

function BGC.RaidID(name)
	return PD.raidID[name]
end

function BGC.UpdateHealer(map)
	-- TODO: Update the onhover/frame
end

function BGC.UpdateDamage(map)
	-- TODO: Update the onhover/frame
end

-- TODO: Timer tick to "stale" old data

function BGC.PlayerName(name,realm)
	return name.."-"..realm
end

function BGC.SpellRange(id)
	local _, _, _, _, _, maxRange = GetSpellInfo(id)
	return maxRange
end

-- We need to do this, name doesn't include own realm and
--	guid is really long to compare... 
-- TODO: Benchmark GUID compare vs GetPlayerInfoByGUID()
function BGC.PlayerNameFromGUID(guid)
	local _,_,_,_,_,name,realm = GetPlayerInfoByGUID(from);
	return BGC.PlayerName(name,realm)
end

function BGC.IsHostile(flags)
	return bit.band(flags, COMBATLOG_OBJECT_REACTION_HOSTILE) > 0
end

function BGC.Addon_Loaded(...)
	local name = ...	
	if "BG_Control" == name then
		BGC.frame:UnregisterEvent("ADDON_LOADED");
		BGC.InitPD();
		hooksecurefunc("LeaveBattlefield",BGControlPublic.LeaveBattlefield);
	end
end

function BGC.OnEvent(self, event, ...)
	if "COMBAT_LOG_EVENT_UNFILTERED" == event then
		BGC.Combat_Log_Event_Unfiltered(...);
	else if "UPDATE_BATTLEFIELD_SCORE" == event then
		BGC.Update_Battlefield_Score(...);
	else if "PLAYER_ENTERING_BATTLEGROUND" == event then
		BGC.Player_Entering_Battleground(...);
	else if "INSTANCE_GROUP_SIZE_CHANGED" = event then
		BGC.Instance_Group_Size_Changed(...)
	else if "ADDON_LOADED" == event then
		BGC.Addon_Loaded(...);
	end
end

BGC.frame:SetScript("OnEvent",BGC.OnEvent);
BGC.frame:RegisterEvent("ADDON_LOADED");
BGC.frame:RegisterEvent("PLAYER_ENTERING_BATTLEGROUND");


